@@ -11,10 +11,18 @@ import redis
 from redis.commands.json.path import Path
 
 from test import testtools
+from test.testtools import resp_conversion, get_protocol_version
 
 json_tests = pytest.importorskip("jsonpath_ng")
 
 
+def test_jsonsetnx(r: redis.Redis):
+    key = "test_key"
+    assert r.json().set("test_key", "$", {}, nx=True, decode_keys=True) is True
+    assert r.json().set(key, "$.group", {}, nx=True) is True
+    assert r.json().set("test_key", "$.group", {"data": "value"}, nx=True) is None
+
+
 def test_jsonget(r: redis.Redis):
     data = {"x": "bar", "y": {"x": 33}}
     r.json().set("foo", Path.root_path(), data)
@@ -270,9 +278,7 @@ def test_jsonstrlen(r: redis.Redis):
     assert r.json().strlen("foo2", Path.root_path()) == 5
 
     r.json().set("foo3", Path.root_path(), {"x": "string"})
-    assert r.json().strlen("foo3", Path("$.x")) == [
-        6,
-    ]
+    assert r.json().strlen("foo3", Path("$.x")) == [6]
 
     assert r.json().strlen("non-existing") is None
 
@@ -435,14 +441,7 @@ def test_decode_response_disabaled_null(r: redis.Redis):
 
 
 def test_json_get_jset(r: redis.Redis):
-    assert (
-        r.json().set(
-            "foo",
-            Path.root_path(),
-            "bar",
-        )
-        == 1
-    )
+    assert r.json().set("foo", Path.root_path(), "bar") == 1
     assert "bar" == r.json().get("foo")
     assert r.json().get("baz") is None
     assert 1 == r.json().delete("foo")
@@ -508,21 +507,12 @@ def test_set_path(r: redis.Redis):
     assert r.json().get(jsonfile.name.rsplit(".")[0]) == {"hello": "world"}
 
 
+@pytest.mark.min_server("7")
 def test_type(r: redis.Redis):
-    r.json().set(
-        "1",
-        Path.root_path(),
-        1,
-    )
+    r.json().set("1", Path.root_path(), 1)
 
-    assert (
-        r.json().type(
-            "1",
-            Path.root_path(),
-        )
-        == b"integer"
-    )
-    assert r.json().type("1") == b"integer"  # noqa: E721
+    assert r.json().type("1", Path.root_path()) == resp_conversion(r, [b"integer"], b"integer")
+    assert r.json().type("1") == resp_conversion(r, [b"integer"], b"integer")  # noqa: E721
 
     meta_data = {
         "object": {},
@@ -536,14 +526,18 @@ def test_type(r: redis.Redis):
     data = {k: {"a": meta_data[k]} for k in meta_data}
     r.json().set("doc1", "$", data)
     # Test multi
-    assert r.json().type("doc1", "$..a") == [k.encode() for k in meta_data.keys()]  # noqa: E721
+    expected = [k.encode() for k in meta_data.keys()]
+    assert r.json().type("doc1", "$..a") == resp_conversion(r, [expected], expected)  # noqa: E721
 
     # Test single
-    assert r.json().type("doc1", "$.integer.a") == [b"integer"]  # noqa: E721
-    assert r.json().type("doc1") == b"object"  # noqa: E721
+    assert r.json().type("doc1", "$.integer.a") == resp_conversion(r, [[b"integer"]], [b"integer"])
+    assert r.json().type("doc1") == resp_conversion(r, [b"object"], b"object")
 
     # Test missing key
-    assert r.json().type("non_existing_doc", "..a") is None
+    if get_protocol_version(r) == 2:
+        assert r.json().type("non_existing_doc", "..a") is None
+    else:
+        assert r.json().type("non_existing_doc", "..a") == [None]
 
 
 def test_objlen(r: redis.Redis):
@@ -553,15 +547,8 @@ def test_objlen(r: redis.Redis):
 
     obj = {"foo": "bar", "baz": "qaz"}
 
-    r.json().set(
-        "obj",
-        Path.root_path(),
-        obj,
-    )
-    assert len(obj) == r.json().objlen(
-        "obj",
-        Path.root_path(),
-    )
+    r.json().set("obj", Path.root_path(), obj)
+    assert len(obj) == r.json().objlen("obj", Path.root_path())
 
     r.json().set("obj", Path.root_path(), obj)
     assert len(obj) == r.json().objlen("obj")
@@ -600,12 +587,12 @@ def test_objkeys(r: redis.Redis):
     r.json().set("obj", Path.root_path(), obj)
     keys = r.json().objkeys("obj", Path.root_path())
     keys.sort()
-    exp = list(obj.keys())
+    exp = [k.encode() for k in obj.keys()]
     exp.sort()
-    assert exp == keys
+    assert set(keys) == resp_conversion(r, {k.encode() for k in obj.keys()}, set(obj.keys()))
 
     r.json().set("obj", Path.root_path(), obj)
-    assert r.json().objkeys("obj") == list(obj.keys())
+    assert set(r.json().objkeys("obj")) == resp_conversion(r, {k.encode() for k in obj.keys()}, set(obj.keys()))
 
     assert r.json().objkeys("fakekey") is None
 
@@ -623,9 +610,9 @@ def test_objkeys(r: redis.Redis):
     assert r.json().objkeys("doc1", "$.nested1.a") == [[b"foo", b"bar"]]
 
     # Test legacy
-    assert r.json().objkeys("doc1", ".*.a") == ["foo", "bar"]
+    assert set(r.json().objkeys("doc1", ".*.a")) == resp_conversion(r, {b"foo", b"bar"}, {"foo", "bar"})
     # Test single
-    assert r.json().objkeys("doc1", ".nested2.a") == ["baz"]
+    assert r.json().objkeys("doc1", ".nested2.a") == resp_conversion(r, [b"baz"], ["baz"])
 
     # Test missing key
     assert r.json().objkeys("non_existing_doc", "..a") is None
@@ -637,12 +624,13 @@ def test_objkeys(r: redis.Redis):
     assert r.json().objkeys("doc1", "$..nowhere") == []
 
 
+@pytest.mark.min_server("7")
 def test_numincrby(r: redis.Redis):
     r.json().set("num", Path.root_path(), 1)
 
-    assert 2 == r.json().numincrby("num", Path.root_path(), 1)
-    assert 2.5 == r.json().numincrby("num", Path.root_path(), 0.5)
-    assert 1.25 == r.json().numincrby("num", Path.root_path(), -1.25)
+    assert r.json().numincrby("num", Path.root_path(), 1) == resp_conversion(r, [2], 2)
+    assert r.json().numincrby("num", Path.root_path(), 0.5) == resp_conversion(r, [2.5], 2.5)
+    assert r.json().numincrby("num", Path.root_path(), -1.25) == resp_conversion(r, [1.25], 1.25)
     # Test NUMINCRBY
     r.json().set("doc1", "$", {"a": "b", "b": [{"a": 2}, {"a": 5.0}, {"a": "c"}]})
     # Test multi
@@ -656,13 +644,14 @@ def test_numincrby(r: redis.Redis):
     assert r.json().numincrby("doc1", "$.b[1].a", 3.5) == [15.0]
 
 
+@pytest.mark.min_server("7")
 def test_nummultby(r: redis.Redis):
     r.json().set("num", Path.root_path(), 1)
 
     with pytest.deprecated_call():
-        assert r.json().nummultby("num", Path.root_path(), 2) == 2
-        assert r.json().nummultby("num", Path.root_path(), 2.5) == 5
-        assert r.json().nummultby("num", Path.root_path(), 0.5) == 2.5
+        assert r.json().nummultby("num", Path.root_path(), 2) == resp_conversion(r, [2], 2)
+        assert r.json().nummultby("num", Path.root_path(), 2.5) == resp_conversion(r, [5], 5)
+        assert r.json().nummultby("num", Path.root_path(), 0.5) == resp_conversion(r, [2.5], 2.5)
 
     r.json().set("doc1", "$", {"a": "b", "b": [{"a": 2}, {"a": 5.0}, {"a": "c"}]})
 
@@ -684,13 +673,13 @@ def test_nummultby(r: redis.Redis):
 
     # Test legacy NUMINCRBY
     r.json().set("doc1", "$", {"a": "b", "b": [{"a": 2}, {"a": 5.0}, {"a": "c"}]})
-    assert r.json().numincrby("doc1", ".b[0].a", 3) == 5
+    assert r.json().numincrby("doc1", ".b[0].a", 3) == resp_conversion(r, [5], 5)
 
     # Test legacy NUMMULTBY
     r.json().set("doc1", "$", {"a": "b", "b": [{"a": 2}, {"a": 5.0}, {"a": "c"}]})
 
     with pytest.deprecated_call():
-        assert r.json().nummultby("doc1", ".b[0].a", 3) == 6
+        assert r.json().nummultby("doc1", ".b[0].a", 3) == resp_conversion(r, [6], 6)
 
 
 @testtools.run_test_if_redispy_ver("gte", "4.6")
